//
//  ENPopover.h
//  ENPopover
//
//  Created by Brandon Volbright on 12/1/11.
//  Copyright (c) 2011 Evernote. All rights reserved.
//

#import <Foundation/Foundation.h>

// The notification's object is the sending ENPopover.
extern NSString * const ENPopoverWillShowNotification; // sent before the popover is shown
extern NSString * const ENPopoverWillCloseNotification; // sent when the popover is closed, probably mostly useful for ENPopover in 10.6(?)
extern NSString * const ENPopoverDeallocNotification; // sent when the popover is dealloced, probably mostly useful for ENPopover in 10.6(?)


@class MAAttachedWindow;


/**
 * Abstraction layer over NSPopover (OS X 10.7+) and MAAttachedWindow (OS X 10.6-).
 *
 * Usage: create an ENPopover (init), assign it a view controller which contains a view,
 * show it (showRelativeToRect:ofView:preferredEdge:), close it.
 *
 */
@interface ENPopover : NSObject {
  NSUInteger mMode;
  NSViewController * mContentViewController;
  NSPopover * mPopover;
  MAAttachedWindow * mAttachedWindow;
  BOOL mShown;
  NSPopoverAppearance mAttachedWindowAppearance; // only used on 10.6
  NSPopoverBehavior mAttachedWindowBehavior;     // only used on 10.6
  NSColor *mBackgroundColor;
}

/**
 * Returns whether or not NSPopover should be used for newly created popovers. Returns NO on 10.6.
 */
+ (BOOL)shouldUseNSPopover;


- (id)initAsAttachedWindow;


/**
 * Only useful on 10.7.
 * To get notified when a popover is closing on 10.6, observe ENPopoverClosedNotification instead.
 */
@property (assign) id NSPopoverDelegate;

/**
 * A view controller for the view to be displayed in the popover.
 * This property must be set before the popover is displayed, and the view controller must
 * have a non-nil view.
 */
@property (retain) NSViewController * contentViewController;


/**
 * Whether the popover currently is being displayed.
 */
@property (readonly) BOOL shown;


/**
 * Close behavior of the popover.
 * On 10.6, NSPopoverBehaviorTransient is supported only if the parent window
 * supports it by implementing the ENPopoverAware protocol (see also ENPopoverAwareWindow).
 * Specifying NSPopoverBehaviorTransient and attempting to show the popover in a window
 * that is not ENPopoverAware will cause an exception.
 */
@property (assign) NSPopoverBehavior behavior;


/**
 * Whether the popover animates.
 * Only supported on 10.7. On 10.6 it's always NO.
 */
@property (assign) BOOL animates;


/**
 * Sets the appearance of the popover.
 * HUD appearance is supported on 10.6 by tweaking the MAAttachedWindow's colors manually.
 * This property is overridden when backgroundColor is not nil.
 */
@property (assign) NSPopoverAppearance appearance;


/**
 * Set the background color of a popover.
 * We get this for free on 10.6 with MAAttachedWindow, but on 10.7, the popover's content view must
 * be a subclass of ENCustomPopoverBackgroundView, since we use a (legal!) hack to override NSPopover's background. 
 * Set to nil to use the default popover background, specified by the appearance property.
 */
@property (retain) NSColor *backgroundColor;


/**
 * The size of the view inside the popover.
 * Can be modified to dynamically change the size of the popover.
 */
@property (assign) NSSize contentSize;


/**
 * Returns YES if the receiver is using an NSPopover.
 */
@property (readonly) BOOL usesNSPopover;

/**
 * This method mimics its NSPopover namesake.
 * @param positioningRect The rectangle within positioningView relative to which the popover 
 * should be positioned. Normally set to the bounds of positioningView. 
 * @param positioningView The view relative to which the popover should be positioned. 
 * May not be nil.
 * @param preferredEdge The edge of positioningView the popover should prefer to be anchored to.
 */
- (void)showRelativeToRect:(NSRect)positioningRect 
                    ofView:(NSView *)positioningView
             preferredEdge:(NSRectEdge)preferredEdge;

/**
 * This method mimics its NSPopover namesake.
 * Forces the popover to close.
 */
- (void)close;

/**
 * Returns a new, autoreleased ENPopover with a view controller set up, and an
 * ENCustomBackgroundPopoverView as the content view.
 */
+ (ENPopover *)customBackgroundPopoverWithContentSize:(NSSize)size;

/**
 * If this popover uses an ENCustomBackgroundPopoverView as the content view,
 * returns the ENCustomBackgroundPopoverView's contentView.
 * Any controls/subviews should be added to *this* view, and *not* [contentViewController view],
 * because of the hacks that ENCustomBackgroundPopoverView uses to tint the popover background.
 */
- (NSView *)customBackgroundContentView;

@end
