//
//  ENAudioRecorder.h
//  EverNote
//
//  Created by Brandon Volbright on 3/22/11.
//  Copyright 2011 Evernote. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#import <AudioToolbox/AudioToolbox.h>

@class QTCaptureDevice;
@class QTCaptureDeviceInput;
@class QTCaptureOutput;
@class QTCaptureSession;
@class ENAudioRecorder;

@protocol ENAudioRecorderDelegate <NSObject>
@optional
- (void)recorderHaltedBecauseMaxFileSizeReached:(ENAudioRecorder *)audioRecorder;
- (void)recordingSaved:(ENAudioRecorder *)audioRecorder success:(BOOL)success;
- (void)recordingDiscarded:(ENAudioRecorder *)audioRecorder;
@end


@interface ENAudioRecorder : NSObject {
  CGFloat mAudioLevel;
  void * mRecordingState;
  BOOL mListeningToAudioSource;
  NSTimer * mTimer;
  NSError * mError;
  BOOL mRecording;
  NSURL * mFileURL;
  NSInteger mMaximumFileSize;
  NSTimeInterval mRecordingStartTime;
  NSTimeInterval mRecordedInterval;
  id<ENAudioRecorderDelegate> mDelegate;
  QTCaptureSession * mCaptureSession;
  QTCaptureDevice * mDevice;
  QTCaptureDeviceInput * mDeviceInput;
  QTCaptureOutput * mOutput;
}


@property (readonly) CGFloat audioLevel; // 0.0 - 1.0
@property (readonly) BOOL isListeningToAudioSource;
@property (readonly) BOOL isRecording;
@property (readonly,retain) NSError * mostRecentError;
@property (readonly,retain) NSURL * audioFileURL;
@property (nonatomic, assign) NSInteger maximumFileSize;
@property (readonly) NSTimeInterval recordedInterval;
@property (assign) id<ENAudioRecorderDelegate> delegate;


/**
 * Start listening to audio data on the system's current input device.
 * The audioLevel property will start indicating meaningful values after this call (if successful).
 * Returns YES if successful; if NO, check the mostRecentError property for details.
 */
- (BOOL) startListeningToAudioSource;


/**
 * Stop listening to audio data.
 * The audioLevel property will stop indicating meaningful values after this call.
 * Returns YES if successful; if NO, check the mostRecentError property for details.
 */
- (BOOL) stopListeningToAudioSource;


/**
 * Start recording audio data to a file.
 * After a successful call, the location of the audio data file can be found in the
 * audioFileURL property.
 * @param audioFolder  The folder where the recording should be stored
 * Returns YES if successful; if NO, check the mostRecentError property for details.
 */
- (BOOL) startRecordingInDirectory:(NSString *)audioDirectory;


/**
 * Stop recording audio data to file and perform any file cleanup necessary.
 * After a successful call, the location of the audio data file can be found in the
 * audioFileURL property.
 * Returns YES if successful; if NO, check the mostRecentError property for details.
 */
- (BOOL) saveRecording;


/**
 * Stop recording audio data to file and discard the file.
 * Returns YES if successful; if NO, check the mostRecentError property for details.
 */
- (BOOL) discardRecording;


/**
 * Extension used for audio recording files.
 */
+ (NSString *) audioFileExtension;


/**
 * Repair the audio file at path to make it playable.
 * Returns YES on success.
 */
- (BOOL) repairInterruptedAudioFile: (NSString *) path;


@end



@interface NSObject (ENAudioRecorderDelegate)
- (void) recorderHaltedBecauseMaxFileSizeReached: (ENAudioRecorder *) recorder;
- (void) recordingDiscarded: (ENAudioRecorder *) recorder;
- (void) recordingSaved: (ENAudioRecorder *) recorder success: (BOOL) success;
@end
