//
// ENQuickNote
// Evernote
//
// Created by Ruben Bakker on 1/21/13.
// Copyright (c) 2012 Evernote. All rights reserved.
//

#import <Foundation/Foundation.h>

@class ENQuickNote;
@class CLLocation;

extern NSString *const kENQuickNoteAttachmentMarker;

typedef void (^ENQuickNoteEnumerationBlock)(ENQuickNote *quickNote, BOOL *stop);

/**
 * This class stores all files belonging to a quick note.
 * It's used both in the EvernoteHelper and Evernote main bundle.
 */
@interface ENQuickNote : NSObject <NSCoding> {
  NSString *mLoginIDString;
  NSURL *mPathURL;
  NSAttributedString *mText;
  NSDate *mSavedDate;
  CLLocation *mSavedLocation;
  NSMutableDictionary *mAttachments;
  BOOL mIsGeolocating;
  BOOL mIgnoreSave;
}

/** The account belonging to the user */
@property (retain, readonly) NSString *loginIDString;

/** The path to a directory holding quick note data */
@property (retain, readonly) NSURL *pathURL;

/** The text */
@property (retain) NSAttributedString *text;

/** When was the note created? */
@property (retain, readonly) NSDate *savedDate;

/** Where was this note created */
@property (retain, readonly) CLLocation *savedLocation;

/**
 * Convert usernameOrEmail and bootstrapProfileName to a single string
 */
+ (NSString *)loginIDFromUsernameOrEmail:(NSString *)usernameOrEmail andBootstrapProfileName:(NSString *)bootstrapProfileName;

/**
 * Create a quick note and persist it. Make sure to use 'cleanUp' to remove the files again.
 */
+ (ENQuickNote *)quickNoteForLoginIDString:(NSString *)loginIDString;

/**
 * For each persisted ENQuickNote call block
 */
+ (void)enumeratePendingQuickNotesForLoginIDString:(NSString *)loginIDString withBlock:(ENQuickNoteEnumerationBlock)enumerationBlock;

/**
 * Get the note currently being edited from the storage on disk.
 * @return the preserved quick note
 */
+ (ENQuickNote *)currentPreservedQuickNoteForLoginIDString:(NSString *)loginIDString;

/**
 * @return the directory for all quick notes
 */
+ (NSURL *)quickNoteRootDirectory;

/**
 * Add an attachment. The attachment is stored in the file system.
 * @return the thumbnail for the attachment used to be included into the text
 */
- (NSURL *)addAttachment:(NSURL *)attachmentPath;

/**
 * Add an attachment from existing NSFileWrapper
 */
- (NSURL *)addAttachmentFromWrapper:(NSFileWrapper *)fileWrapper;

/**
 * @return the text with the real attachments (not the thumbnails).
 */
- (NSAttributedString *)textWithOriginalAttachments:(NSArray **)outReplacedAttachments;

/**
 * Tag the note with current date and user location.
 */
- (void)tagIfNeeded;

/**
 * Persist the object in its pathURL folder.
 */
- (BOOL)save:(NSError **)outError;

/**
 * Save note, but make sure it's not processed by the main app.
 */
- (BOOL)preserve:(NSError **)outError;

/**
 * Creates a new temporary directory inside the quick note dir.
 * The template needs some capital 'X' chars to make it unique (see mkdtemp doc).
 * Example: images.XXXXXX
 */
- (NSURL *)createContainerDirWithTemplate:(NSString *)template;

/**
 * Remove all files that belong to this quick note.
 */
- (void)cleanUp;


@end